@@ -1,6 +1,6 @@
-;;; company-abbrev.el --- company-mode completion backend for abbrev
+;;; company-abbrev.el --- company-mode completion backend for abbrev  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2009-2011, 2013-2015, 2021  Free Software Foundation, Inc.
+;; Copyright (C) 2009-2011, 2013-2015, 2021, 2023  Free Software Foundation, Inc.
 
 ;; Author: Nikolaj Schumacher
 
@@ -29,21 +29,23 @@
 (require 'cl-lib)
 (require 'abbrev)
 
-(defun company-abbrev-insert (match)
+(defun company-abbrev-insert (_match)
   "Replace MATCH with the expanded abbrev."
   (expand-abbrev))
 
 ;;;###autoload
-(defun company-abbrev (command &optional arg &rest ignored)
+(defun company-abbrev (command &optional arg &rest _ignored)
   "`company-mode' completion backend for abbrev."
   (interactive (list 'interactive))
   (cl-case command
     (interactive (company-begin-backend 'company-abbrev
                                         'company-abbrev-insert))
     (prefix (company-grab-symbol))
-    (candidates (nconc
-                 (delete "" (all-completions arg global-abbrev-table))
-                 (delete "" (all-completions arg local-abbrev-table))))
+    (candidates (apply
+                 #'nconc
+                 (mapcar (lambda (table)
+                           (delete "" (all-completions arg table)))
+                         (abbrev--active-tables))))
     (kind 'snippet)
     (meta (abbrev-expansion arg))
     (post-completion (expand-abbrev))))
