@@ -1,6 +1,6 @@
 ;;; core-tests.el --- company-mode tests  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2018, 2020-2022  Free Software Foundation, Inc.
+;; Copyright (C) 2015-2018, 2020-2024  Free Software Foundation, Inc.
 
 ;; Author: Dmitry Gutov
 
@@ -22,16 +22,12 @@
 (require 'company-tests)
 
 (ert-deftest company-good-prefix ()
-  (let ((company-minimum-prefix-length 5)
-        company-abort-manual-when-too-short
-        company--manual-action            ;idle begin
-        (company-selection-changed t))    ;has no effect
-    (should (eq t (company--good-prefix-p "!@#$%")))
-    (should (eq nil (company--good-prefix-p "abcd")))
-    (should (eq nil (company--good-prefix-p 'stop)))
-    (should (eq t (company--good-prefix-p '("foo" . 5))))
-    (should (eq nil (company--good-prefix-p '("foo" . 4))))
-    (should (eq t (company--good-prefix-p '("foo" . t))))))
+  (should (eq t (company--good-prefix-p "!@#$%" 5)))
+  (should (eq nil (company--good-prefix-p "abcd" 5)))
+  (should (eq nil (company--good-prefix-p 'stop 5)))
+  (should (eq t (company--good-prefix-p '("foo" . 5) 5)))
+  (should (eq nil (company--good-prefix-p '("foo" . 4) 5)))
+  (should (eq t (company--good-prefix-p '("foo" . t) 5))))
 
 (ert-deftest company--manual-prefix-set-and-unset ()
   (with-temp-buffer
@@ -39,7 +35,7 @@
     (company-mode)
     (let (company-frontends
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abc" "abd")))))))
@@ -55,7 +51,7 @@
     (let (company-frontends
           (company-abort-on-unique-match t)
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abc")))))))
@@ -69,7 +65,7 @@
     (let (company-frontends
           company-abort-on-unique-match
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abc")))))))
@@ -82,30 +78,28 @@
     (company-mode)
     (let (company-frontends
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abc")))))))
       (company-manual-begin)
       (should (equal '("abc") company-candidates)))))
 
-(ert-deftest company-abort-manual-when-too-short ()
+(ert-deftest company-prefix-min-length ()
   (let ((company-minimum-prefix-length 5)
-        (company-abort-manual-when-too-short t)
         (company-selection-changed t))    ;has not effect
-    (let ((company--manual-action nil))   ;idle begin
-      (should (eq t (company--good-prefix-p "!@#$%")))
-      (should (eq t (company--good-prefix-p '("foo" . 5))))
-      (should (eq t (company--good-prefix-p '("foo" . t)))))
-    (let ((company--manual-action t)
+    (should (= (company--prefix-min-length) 5))
+    (let ((company-abort-manual-when-too-short t)
           (company--manual-prefix "abc")) ;manual begin from this prefix
-      (should (eq t (company--good-prefix-p "!@#$")))
-      (should (eq nil (company--good-prefix-p "ab")))
-      (should (eq nil (company--good-prefix-p 'stop)))
-      (should (eq t (company--good-prefix-p '("foo" . 4))))
-      (should (eq t (company--good-prefix-p "abcd")))
-      (should (eq t (company--good-prefix-p "abc")))
-      (should (eq t (company--good-prefix-p '("bar" . t)))))))
+      (should (= (company--prefix-min-length) 3)))
+    (let ((company-abort-manual-when-too-short t)
+          (company--manual-prefix '("abc" . 2)))
+      (should (= (company--prefix-min-length) 2)))
+    (let ((company-abort-manual-when-too-short t)
+          (company--manual-prefix '("abc" . t)))
+      (should (= (company--prefix-min-length) 3)))
+    (let ((company--manual-prefix "abc"))
+      (should (= (company--prefix-min-length) 0)))))
 
 (ert-deftest company-common-with-non-prefix-completion ()
   (let ((company-backend #'ignore)
@@ -115,9 +109,9 @@
         company-candidates-cache
         company-common)
     (company-update-candidates '("abc" "def-abc"))
-    (should (null company-common))
+    (should (equal company-common ""))
     (company-update-candidates '("abc" "abe-c"))
-    (should (null company-common))
+    (should (equal company-common "ab"))
     (company-update-candidates '("abcd" "abcde" "abcdf"))
     (should (equal "abcd" company-common))))
 
@@ -131,44 +125,66 @@
                  (cl-case command
                    (prefix "z")
                    (candidates '("c" "d")))))))
+    (company-call-backend 'set-min-prefix 1)
     (should (equal (company-call-backend 'candidates "z") '("a" "b" "c" "d")))))
 
-(ert-deftest company-multi-backend-filters-backends-by-prefix ()
+(ert-deftest company-multi-backend-with-empty-prefixes ()
   (let ((company-backend
          (list (lambda (command &optional _ &rest _r)
                  (cl-case command
-                   (prefix (cons "z" t))
+                   (prefix "")
                    (candidates '("a" "b"))))
                (lambda (command &optional _ &rest _r)
                  (cl-case command
-                   (prefix "t")
-                   (candidates '("c" "d"))))
-               (lambda (command &optional _ &rest _r)
+                   (prefix "")
+                   (candidates '("c" "d")))))))
+    (should (equal (company-call-backend 'prefix) '("" nil 0)))))
+
+(ert-deftest company-multi-backend-none-applicable ()
+  (let ((company-backend (list #'ignore #'ignore)))
+    (should (null (company-call-backend 'prefix)))))
+
+(ert-deftest company-multi-backend-dispatches-separate-prefix-to-backends ()
+  (let ((company-backend
+         (list (lambda (command &optional arg &rest _r)
                  (cl-case command
-                   (prefix "z")
-                   (candidates '("e" "f")))))))
-    (should (equal (company-call-backend 'candidates "z") '("a" "b" "e" "f")))))
+                   (prefix (cons "z" t))
+                   (candidates
+                    (should (equal arg "z"))
+                    '("a" "b"))))
+               (lambda (command &optional arg &rest _r)
+                 (cl-case command
+                   (prefix "t")
+                   (candidates
+                    (should (equal arg "t"))
+                    '("c" "d")))))))
+    (company-call-backend 'set-min-prefix 1)
+    (should (equal (company-call-backend 'candidates "z") '("a" "b" "c" "d")))))
 
 (ert-deftest company-multi-backend-remembers-candidate-backend ()
   (let ((company-backend
-         (list (lambda (command &optional _)
+         (list (lambda (command &rest _)
                  (cl-case command
+                   (prefix "")
                    (ignore-case nil)
                    (annotation "1")
                    (candidates '("a" "c"))
                    (post-completion "13")))
-               (lambda (command &optional _)
+               (lambda (command &rest _)
                  (cl-case command
+                   (prefix "")
                    (ignore-case t)
                    (annotation "2")
                    (candidates '("b" "d"))
                    (post-completion "42")))
-               (lambda (command &optional _)
+               (lambda (command &rest _)
                  (cl-case command
+                   (prefix "")
                    (annotation "3")
                    (candidates '("e"))
                    (post-completion "74"))))))
-    (let ((candidates (company-calculate-candidates nil nil)))
+    (company-call-backend 'set-min-prefix 0)
+    (let ((candidates (company-calculate-candidates "" nil nil)))
       (should (equal candidates '("a" "b" "c" "d" "e")))
       (should (equal t (company-call-backend 'ignore-case)))
       (should (equal "1" (company-call-backend 'annotation (nth 0 candidates))))
@@ -179,31 +195,32 @@
       (should (equal "74" (company-call-backend 'post-completion (nth 4 candidates)))))))
 
 (ert-deftest company-multi-backend-handles-keyword-with ()
-  (let ((primo (lambda (command &optional _)
+  (let ((primo (lambda (command &rest _)
                  (cl-case command
                    (prefix "a")
                    (candidates '("abb" "abc" "abd")))))
-        (secundo (lambda (command &optional _)
+        (secundo (lambda (command &rest _)
                    (cl-case command
                      (prefix "a")
                      (candidates '("acc" "acd"))))))
     (let ((company-backend (list 'ignore 'ignore :with secundo)))
       (should (null (company-call-backend 'prefix))))
     (let ((company-backend (list 'ignore primo :with secundo)))
-      (should (equal "a" (company-call-backend 'prefix)))
+      (should (equal '("a" nil 1) (company-call-backend 'prefix)))
+      (company-call-backend 'set-min-prefix 1)
       (should (equal '("abb" "abc" "abd" "acc" "acd")
                      (company-call-backend 'candidates "a"))))))
 
 (ert-deftest company-multi-backend-handles-keyword-separate ()
-  (let ((one (lambda (command &optional _)
+  (let ((one (lambda (command &rest _)
                (cl-case command
                  (prefix "a")
                  (candidates (list "aa" "ca" "ba")))))
-        (two (lambda (command &optional _)
+        (two (lambda (command &rest _)
                (cl-case command
                  (prefix "a")
                  (candidates (list "bb" "ab")))))
-        (tri (lambda (command &optional _)
+        (tri (lambda (command &rest _)
                (cl-case command
                  (prefix "a")
                  (sorted t)
@@ -211,9 +228,203 @@
     (let ((company-backend (list one two tri :separate)))
       (should (company-call-backend 'sorted))
       (should-not (company-call-backend 'duplicates))
+      (company-call-backend 'set-min-prefix 1)
       (should (equal '("aa" "ba" "ca" "ab" "bb" "cc" "bc" "ac")
                      (company-call-backend 'candidates "a"))))))
 
+(ert-deftest company-multi-backend-handles-length-overrides-separately ()
+  (let ((one (lambda (command &rest _)
+               (cl-case command
+                 (prefix "a")
+                 (candidates (list "aa" "ca" "ba")))))
+        (two (lambda (command &rest _)
+               (cl-case command
+                 (prefix (cons "a" 2))
+                 (candidates (list "bb" "ab")))))
+        (tri (lambda (command &rest _)
+               (cl-case command
+                 (prefix "")
+                 (candidates (list "cc" "bc" "ac"))))))
+    (company-call-backend 'set-min-prefix 2)
+    (let ((company-backend (list one two tri)))
+      (should (equal '("bb" "ab")
+                     (company-call-backend 'candidates "a"))))
+    (company-call-backend 'set-min-prefix 1)
+    (let ((company-backend (list one two tri)))
+      (should (equal '("aa" "ca" "ba" "bb" "ab")
+                     (company-call-backend 'candidates "a"))))))
+
+(ert-deftest company-multi-backend-handles-clears-cache-when-needed ()
+  (let* ((one (lambda (command &rest _)
+                (cl-case command
+                  (prefix "aa")
+                  (candidates (list "aa")))))
+         (two (lambda (command &rest _)
+                (cl-case command
+                  (prefix (cons "aa" t))
+                  (candidates (list "aab" )))))
+         (tri (lambda (command &rest _)
+                (cl-case command
+                  (prefix "")
+                  (candidates (list "aac")))))
+         (company--multi-uncached-backends (list one tri)))
+    (let ((company-backend (list one two tri)))
+      (company-call-backend 'set-min-prefix 2)
+      (should
+       (equal (company-call-backend 'no-cache) t))
+      (should (equal company--multi-uncached-backends (list tri)))
+      (should (equal '("aa" "aab")
+                     (company-call-backend 'candidates "aa"))))))
+
+(ert-deftest company-multi-backend-chooses-longest-prefix-length ()
+  (let* ((one (lambda (command &rest _)
+                (cl-case command
+                  (prefix "aa")
+                  (candidates (list "aa")))))
+         (two (lambda (command &rest _)
+                (cl-case command
+                  (prefix (cons "aa" t))
+                  (candidates (list "aab" )))))
+         (tri (lambda (command &rest _)
+                (cl-case command
+                  (prefix "")
+                  (candidates (list "aac")))))
+         (fur (lambda (command &rest _)
+                (cl-case command
+                  (prefix (cons "aa" 3))
+                  (candidates (list "aac")))))
+         (fiv (lambda (command &rest _)
+                (cl-case command
+                  (prefix (cons "aa" 1))
+                  (candidates (list "aac")))))
+         (company--multi-uncached-backends (list one tri)))
+    (let ((company-backend (list one tri fur)))
+      (should
+       (equal
+        '("aa" nil 3)
+        (company-call-backend 'prefix))))
+    (let ((company-backend (list one two tri fur)))
+      (should
+       (equal
+        '("aa" nil t)
+        (company-call-backend 'prefix))))
+    (let ((company-backend (list one fiv)))
+      (should
+       (equal
+        '("aa" nil 2)
+        (company-call-backend 'prefix))))))
+
+(ert-deftest company-multi-backend-supports-different-suffixes ()
+  (let* ((one (lambda (command &rest args)
+                (cl-case command
+                  (prefix '("a" "b"))
+                  (candidates
+                   (should (equal args '("a" "b")))
+                   '("a1b")))))
+         (two (lambda (command &rest args)
+                (cl-case command
+                  (prefix "a")
+                  (candidates
+                   (should (equal args '("a" "")))
+                   '("a2")))))
+         (tri (lambda (command &rest args)
+                (cl-case command
+                  (prefix '("a" ""))
+                  (candidates
+                   (should (equal args '("a" "")))
+                   '("a3")))))
+         (company-backend (list one two tri)))
+    (should
+     (equal '("a" "b" 1)
+            (company-call-backend 'prefix)))
+    (should
+     (equal '("a1b" "a2" "a3")
+            (company-call-backend 'candidates "a" "b")))))
+
+(ert-deftest company-multi-backend-dispatches-adjust-boundaries ()
+  (let* ((one (lambda (command &rest _args)
+                (cl-case command
+                  (prefix '("a" ""))
+                  (candidates
+                   '("a1b")))))
+         (tri (lambda (command &rest args)
+                (cl-case command
+                  (prefix '("aa" "bcd"))
+                  (adjust-boundaries
+                   (should (equal args
+                                  '("a3" "aa" "bcd")))
+                   (cons "a" "bc"))
+                  (candidates
+                   '("a3")))))
+         (company-backend (list one tri))
+         (company-point (point))
+         (candidates (company-call-backend 'candidates "a" "")))
+    (should
+     (equal '("aa" "bcd" 2)
+            (company-call-backend 'prefix)))
+    (should
+     (equal (cons "a" "bc")
+            (company-call-backend 'adjust-boundaries
+                                  (car (member "a3" candidates))
+                                  "aa" "bcd")))
+    (should
+     (equal (cons "a" "")
+            (company-call-backend 'adjust-boundaries
+                                  (car (member "a1b" candidates))
+                                  "aa" "bcd")))))
+
+(ert-deftest company-multi-backend-combines-expand-common ()
+  (let* ((one (lambda (command &rest _args)
+                (cl-case command
+                  (prefix '("a" ""))
+                  (expand-common (cons "ab" "")))))
+         (two (lambda (command &rest _args)
+                (cl-case command
+                  (prefix '("aa" "bcd"))
+                  (expand-common (cons "aab" "bcd")))))
+         (tri (lambda (command &rest _args)
+                (cl-case command
+                  (prefix '("aa" "bcd"))
+                  (expand-common 'no-match))))
+         (company-backend (list one two tri))
+         (company-point (point)))
+    (company-call-backend 'set-min-prefix 1)
+    (should
+     (equal '("aab" . "bcd")
+            (company-call-backend 'expand-common "aa" "bcd")))))
+
+(ert-deftest company-multi-backend-expand-common-returns-no-match ()
+  (let* ((one (lambda (command &rest _args)
+                (cl-case command
+                  (prefix '("a" ""))
+                  (expand-common 'no-match))))
+         (two (lambda (command &rest _args)
+                (cl-case command
+                  (prefix '("aa" "bcd"))
+                  (expand-common 'no-match))))
+         (company-backend (list one two))
+         (company-point (point)))
+    (company-call-backend 'set-min-prefix 1)
+    (should
+     (equal 'no-match
+            (company-call-backend 'expand-common "aa" "bcd")))))
+
+(ert-deftest company-multi-backend-expand-common-keeps-current ()
+  (let* ((one (lambda (command &rest _args)
+                (cl-case command
+                  (prefix '("a" ""))
+                  (expand-common (cons "ab" "")))))
+         (two (lambda (command &rest _args)
+                (cl-case command
+                  (prefix '("a" ""))
+                  (expand-common (cons "ac" "")))))
+         (company-backend (list one two))
+         (company-point (point)))
+    (company-call-backend 'set-min-prefix 1)
+    (should
+     (equal '("a" . "")
+            (company-call-backend 'expand-common "a" "")))))
+
 (ert-deftest company-begin-backend-failure-doesnt-break-company-backends ()
   (with-temp-buffer
     (insert "a")
@@ -222,7 +433,7 @@
      (company-begin-backend #'ignore))
     (let (company-frontends
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix "a")
                      (candidates '("a" "ab" "ac")))))))
@@ -237,7 +448,7 @@
     (let (company-frontends
           (company-require-match 'company-explicit-action-p)
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abc" "abd")))))))
@@ -256,7 +467,7 @@
           (company-minimum-prefix-length 2)
           (company-require-match 'company-explicit-action-p)
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abc" "abd")))))))
@@ -276,7 +487,7 @@
           company-insertion-on-trigger
           (company-require-match t)
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (company-grab-word))
                      (candidates '("abc" "ab" "abd"))
@@ -299,7 +510,7 @@
     (let (company-frontends
           (company-require-match t)
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (if (> (point) 2)
                                  'stop
@@ -319,7 +530,7 @@
     (let (company-frontends
           company-begin-commands
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abc" "abd")))))))
@@ -341,7 +552,7 @@
     (let (company-frontends
           company-begin-commands
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abc" "abd")))))))
@@ -363,7 +574,7 @@
     (let (company-frontends
           (company-minimum-prefix-length 2)
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abcd" "abef")))))))
@@ -381,7 +592,7 @@
           (company-insertion-on-trigger 'company-explicit-action-p)
           (company-insertion-triggers '(? ))
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abcd" "abef")))))))
@@ -400,7 +611,7 @@
           (company-insertion-on-trigger t)
           (company-insertion-triggers '(? ?\)))
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring 5 (point)))
                      (candidates '("abcd" "abef"))))))
@@ -425,7 +636,7 @@
           (company-insertion-triggers '(? ))
           (company-minimum-prefix-length 2)
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abcd" "abef")))))))
@@ -449,7 +660,7 @@
     (company-mode)
     (let (company-frontends
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abcd" "abef"))
@@ -469,7 +680,7 @@
     (company-mode)
     (let (company-frontends
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("abcd" "abef"))
@@ -485,7 +696,7 @@
     (company-mode)
     (let (company-frontends
           (company-backends
-           (list (lambda (command &optional _)
+           (list (lambda (command &rest _)
                    (cl-case command
                      (prefix (buffer-substring (point-min) (point)))
                      (candidates '("tea-cup" "teal-color")))))))
@@ -735,4 +946,17 @@
     (company-select-next -10)
     (should (eq company-selection 0))))
 
+(ert-deftest company-capf-completions ()
+  (let ((table '("ab-de-b" "ccc" "abc-de-b")))
+    (let ((completion-styles '(partial-completion)))
+      (should
+       (equal (company--capf-completions "ab-d" "b" table)
+              '((:completions . ("ab-de-b" "abc-de-b"))
+                (:boundaries . ("ab-d" . "b"))))))
+    (let ((completion-styles '(emacs22)))
+      (should
+       (equal (company--capf-completions "ab-d" "b" table)
+              '((:completions . ("ab-de-b"))
+                (:boundaries . ("ab-d" . ""))))))))
+
 ;;; core-tests.el ends here.
